__version__ = "0.1.3"

from .datasets.state_of_the_union import StateOfTheUnion
from .datasets.glue import Glue
from .datasets.paul_graham_essay import PaulGrahamEssay
from .datasets.sciq import SciQ
